/***************************************************************************
                          rmultilinedit.cpp  -  description
                             -------------------
    begin                : Mon Sep 27 1999
    copyright            : (C) 1999 by Andreas Mustun
    email                : andrew@ribbonsoft.com
 ***************************************************************************/


/****************************************************************************
** rmultilinedit.cpp 1998/09/19 A. Mustun RibbonSoft
**
** Copyright (C) 1998 RibbonSoft.  All rights reserved.
**
*****************************************************************************/

#include <qmultilinedit.h>
#include <qfiledialog.h>

#include "rmultilinedit.h"
  
#include "rappwin.h"
#include "rfilebase.h"
#include "rfile.h"
#include "rconfig.h"


// Constructor:
//
RMultiLineEdit::RMultiLineEdit(QWidget* _parent,
                               const char* _name)
:QMultiLineEdit(_parent, _name)
{
  setFont(QFont("helvetica", RCONFIG->getSettingInt("Application:FontSize0")));
}


// Destructor:
//
RMultiLineEdit::~RMultiLineEdit()
{
}



// Load a file into this edit field (ask user for a file name):
//
void
RMultiLineEdit::loadFile()
{
  QCString fn;       // File name
  
  fn = QFileDialog::getOpenFileName(0, 0, this);
  if(!fn.isEmpty()) {
    loadFile(fn);
  }
}
    
  
    
// Load a file into this edit field (file name given):
//
bool
RMultiLineEdit::loadFile(const char* _name)
{
  bool ret=false;
  RFileBase file(_name);
  if(file.readFileInBuffer()) {
    if(file.getBuf()) {
      setText((char*)file.getBuf());
      ret=true;
    }
  }
  return ret;
}



// Save the text in a file (ask user for a file name):
//
void
RMultiLineEdit::saveFile()
{
  QCString fn;       // File name
  
  fn = QFileDialog::getSaveFileName(0, 0, this);
  
  if(!fn.isEmpty()) {
    bool usersChoice;

    // No Extension set / append default extension:
    //
    if(fn.find('.')==-1) {
      fn+=".txt";
    }

    QFileInfo fi(fn.data());
             
    if(fi.exists()) {
      usersChoice=fileOverwriteWarning(fn.data(), 
                                       RMES(327),
                                       RAppWin::getRAppWin());
    }
    else {
      usersChoice=true;
    }
                      
    if(!fi.exists() || usersChoice) {
      RFileBase file(fn.data());
      file.copyBufFrom((char*)text().latin1());
      file.saveBufferToFile();
      file.delBuffer();
    }
  }
}

    

// Resize:
//
void
RMultiLineEdit::resizeEvent(QResizeEvent *_ev)
{
  QTableView::resizeEvent( _ev );
}


// EOF






